package swarm

import (
	"fmt"
	"github.com/g8rswimmer/go-twitter/v2"
	"github.com/xueqianLu/twitter-bee/client"
	"golang.org/x/time/rate"
	"sync"
	"time"
)

type ClientWithRateLimiter struct {
	*client.BeeClient
	RateLimit *rate.Limiter
}

type Swarm struct {
	clients    map[string]*ClientWithRateLimiter
	mu         sync.Mutex
	countmu    sync.Mutex
	followermu sync.Mutex
}

var (
	gSwarm *Swarm
)

func GetSwarm() *Swarm {
	if gSwarm == nil {
		gSwarm, _ = InitSwarm(nil)
	}
	return gSwarm
}

func InitSwarm(initialBees []string) (*Swarm, error) {
	s := &Swarm{
		clients: make(map[string]*ClientWithRateLimiter),
	}
	for _, bee := range initialBees {
		s.AddClient(bee)
	}
	gSwarm = s
	return s, nil
}

func (s *Swarm) copyedClients() map[string]*ClientWithRateLimiter {
	s.mu.Lock()
	defer s.mu.Unlock()
	clients := make(map[string]*ClientWithRateLimiter)
	for k, v := range s.clients {
		clients[k] = v
	}
	return clients
}

func (s *Swarm) AddClient(url string) {
	s.mu.Lock()
	defer s.mu.Unlock()
	cli := new(ClientWithRateLimiter)
	cli.BeeClient = client.NewBeeClient(url)
	cli.RateLimit = rate.NewLimiter(rate.Every(15*time.Minute), 40)

	s.clients[url] = cli
}

func (s *Swarm) RemoveClient(url string) {
	s.mu.Lock()
	defer s.mu.Unlock()
	delete(s.clients, url)
}

func (s *Swarm) GetFollowerCount(userID string) (int, error) {
	clients := s.copyedClients()
	s.countmu.Lock()
	defer s.countmu.Unlock()

	for _, cli := range clients {
		res, err := cli.GetFollowerCount(userID)
		if err == nil {
			fmt.Printf("get %v follower count %d\n", userID, res.Count)
			return res.Count, nil
		} else {
			fmt.Println("get follower count failed with err", err.Error())
		}
	}
	return 0, fmt.Errorf("can not get the %v follower count", userID)
}

func (s *Swarm) GetFollowerList(user string, cursor string) ([]*twitter.UserObj, string, *twitter.RateLimit, error) {
	clients := s.copyedClients()

	s.followermu.Lock()
	defer s.followermu.Unlock()

	for _, cli := range clients {
		//if cli.RateLimit.Allow() == false {
		//	continue
		//}
		res, err := cli.GetFollowerList(user, cursor)
		if err == nil {
			list := make([]*twitter.UserObj, 0, len(res.List))
			for _, u := range res.List {
				list = append(list, &twitter.UserObj{
					ID:       u.ID,
					Name:     u.Name,
					UserName: u.UserName,
				})
			}
			return list, res.Next, nil, nil
		} else {
			fmt.Println("get follower list failed with err", err.Error())
		}
	}
	return nil, "", nil, fmt.Errorf("can not get the %v follower list", user)
}
